package fr.univamu.graph;

import java.util.Objects;
import java.util.Random;

public class Edge {
  private final int vertex1;
  private final int vertex2;
  private double weight;

  public Edge(int vertex1, int vertex2) {
    this.vertex1 = vertex1;
    this.vertex2 = vertex2;
    // Random Weight Generation
    Random random = new Random();
    this.weight = random.nextDouble();
  }

  public Edge(int vertex1, int vertex2, double weight) {
    this.vertex1 = vertex1;
    this.vertex2 = vertex2;
    this.weight = weight;
  }

  public int opposite(int vertex) {
    if (vertex == vertex1) return vertex2;
    if (vertex == vertex2) return vertex1;
    throw new IllegalArgumentException(this + ".opposite(" + vertex + ")");
  }

  public int vertex1() {
    return vertex1;
  }

  public int vertex2() {
    return vertex2;
  }

  public double weight() {
    return weight;
  }

  public void setWeight(double weight) {
    this.weight = weight;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this) return true;
    if (obj == null || obj.getClass() != this.getClass()) return false;
    var that = (Edge) obj;
    return this.vertex1 == that.vertex1 &&
            this.vertex2 == that.vertex2 &&
            Objects.equals(this.weight, that.weight);
  }

  @Override
  public int hashCode() {
    return Objects.hash(vertex1, vertex2, weight);
  }

  @Override
  public String toString() {
    return "Edge["
            + vertex1 + ", "
            + vertex2 + ", " +
            + weight + ']';
  }


}

